-- DROP TABLE Registrar CASCADE;
CREATE TABLE Registrar (
  ID SERIAL CONSTRAINT registrar_pkey PRIMARY KEY,
  ICO  varchar(50), -- ICO of registrar
  DIC varchar(50) NOT NULL CHECK(TRIM(dic) != ''), -- DIC of registrar
  varsymb  char(10) CONSTRAINT registrar_varsymb_key UNIQUE, -- coupling variable symbol ( ico )
  VAT boolean NOT NULL DEFAULT True,
  Handle varchar(255) CONSTRAINT registrar_handle_key UNIQUE NOT NULL CHECK(TRIM(handle) != ''),
  Name varchar(1024) NOT NULL CHECK(TRIM(name) != ''),
  Organization varchar(1024) NOT NULL CHECK(TRIM(organization) != ''),
  Street1 varchar(1024) NOT NULL CONSTRAINT registrar_street1_check CHECK(TRIM(street1) != ''),
  -- check if street2 is present OR street3 is not present with respect to the NULL arithmetic
  -- NULL arithmetic: NULL IS TRUE = false
  --                  NULL IS NOT TRUE = true
  Street2 varchar(1024) CONSTRAINT registrar_street23_check CHECK(TRIM(street2) != '' IS TRUE OR
                                                                  TRIM(street3) != '' IS NOT TRUE),
  Street3 varchar(1024),
  City varchar(1024) NOT NULL CHECK(TRIM(city) != ''),
  StateOrProvince varchar(1024),
  PostalCode varchar(32),
  Country char(2) NOT NULL CONSTRAINT registrar_country_fkey REFERENCES enum_country,
  Telephone varchar(32),
  Fax varchar(32),
  Email varchar(1024) NOT NULL CHECK(TRIM(email) != ''),
  Url varchar(1024) NOT NULL CHECK(TRIM(url) != ''),
  System bool NOT NULL default false,
  Regex varchar(30) default NULL,
  is_internal bool NOT NULL default false,
  uuid UUID NOT NULL UNIQUE DEFAULT gen_random_uuid()
);

comment on table Registrar is 'Evidence of registrars, who can create or change administered object via register';
comment on column Registrar.ID is 'unique automatically generated identifier';
comment on column Registrar.ICO is 'organization identification number';
comment on column Registrar.DIC is 'tax identification number';
comment on column Registrar.varsymb is 'coupling variable symbol (ico)';
comment on column Registrar.VAT is 'whether VAT should be counted in invoicing';
comment on column Registrar.Handle is 'unique text string identifying registrar, it is generated by system admin when new registrar is created';
comment on column Registrar.Name is 'registrats name';
comment on column Registrar.Organization is 'Official company name';
comment on column Registrar.Street1 is 'part of address';
comment on column Registrar.Street2 is 'part of address';
comment on column Registrar.Street3 is 'part of address';
comment on column Registrar.City is 'part of address - city';
comment on column Registrar.StateOrProvince is 'part of address - region';
comment on column Registrar.PostalCode is 'part of address - postal code';
comment on column Registrar.Country is 'code for country from enum_country table';
comment on column Registrar.Telephone is 'phone number';
comment on column Registrar.Fax is 'fax number';
comment on column Registrar.Email is 'e-mail address';
comment on column Registrar.Url is 'registrars web address';
comment on column Registrar.Uuid is 'uuid for external reference';

-- DROP TABLE registraracl CASCADE;
CREATE TABLE registraracl (
  id SERIAL CONSTRAINT registraracl_pkey PRIMARY KEY,
  registrarid INTEGER NOT NULL CONSTRAINT registraracl_registrarid_fkey REFERENCES registrar,
  cert VARCHAR(1024) NOT NULL, -- certificate MD5 fingerprint in HEX format 'FA:09:...:73'
  password VARCHAR(1024) NOT NULL, -- password data
  uuid UUID NOT NULL UNIQUE DEFAULT gen_random_uuid(), -- external reference
  create_time TIMESTAMP DEFAULT CURRENT_TIMESTAMP, -- creation time of the record
  cert_data_pem VARCHAR(16384), -- certificate data in PEM format
  CONSTRAINT registraracl_registrarid_cert_fkey UNIQUE (registrarid,cert)
);

CREATE UNIQUE INDEX registraracl_registrarid_hexcert_fkey
       ON registraracl (registrarid, DECODE(REGEXP_REPLACE(cert, ':', '', 'g'), 'hex'));

COMMENT ON TABLE registraracl IS 'Registrars login information';
COMMENT ON COLUMN registraracl.cert IS 'MD5 fingerprint of registrar''s certificate in HEX format ''FA:09:...:73''';
COMMENT ON COLUMN registraracl.password IS 'data generated by password hashing function';
COMMENT ON COLUMN registraracl.uuid IS 'uuid for external reference';
COMMENT ON COLUMN registraracl.create_time IS 'when the record was created';
COMMENT ON COLUMN registraracl.cert_data_pem IS 'certificate data in PEM format';

CREATE TABLE RegistrarInvoice (
  ID SERIAL CONSTRAINT registrarinvoice_pkey PRIMARY KEY,
  RegistrarID INTEGER NOT NULL CONSTRAINT registrarinvoice_registrarid_fkey REFERENCES Registrar, -- registrar id
  Zone integer NOT NULL CONSTRAINT registrarinvoice_zone_fkey REFERENCES Zone,  --  zone for which has registrar an access
  FromDate date NOT NULL , -- date when began registrar work in a zone
  toDate date -- after this date registrar is not allowed to register
);

comment on column RegistrarInvoice.Zone is 'zone for which has registrar an access';
comment on column RegistrarInvoice.FromDate is 'date when began registrar work in a zone';
comment on column RegistrarInvoice.toDate is 'after this date, registrar is not allowed to register';
