/*
 * Copyright (C) 2021  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */
#ifndef __LOG_ENTRY_PROPERTY__E0825DB430A575EC9DD6831DF43455AE
#define __LOG_ENTRY_PROPERTY__E0825DB430A575EC9DD6831DF43455AE

#include "libgrill/strong_type.hh"

#include <map>
#include <vector>

namespace LibGrill {
namespace Logger {

using LogEntryPropertyType = StrongString<struct LogEntryPropertyTypeTag_>;
using LogEntryPropertyValue = StrongString<struct LogEntryPropertyValueTag_>;
using LogEntryPropertyChildren = std::map<LogEntryPropertyType, LogEntryPropertyValue>;

struct LogEntryProperty
{
    LogEntryPropertyType type;
    LogEntryPropertyValue value;
};

struct LogEntryPropertyWithChildren
{
    LogEntryPropertyValue value;
    LogEntryPropertyChildren children;
};

using LogEntryProperties = std::vector<LogEntryPropertyWithChildren>;
using Properties = std::map<LogEntryPropertyType, LogEntryProperties>;

LogEntryPropertyWithChildren& add(Properties&, const LogEntryProperty&);
LogEntryPropertyWithChildren& add(Properties&, const LogEntryPropertyType&, const LogEntryPropertyWithChildren&);

template <typename Type>
LogEntryProperty to_log_entry_property(const Type&);

template <typename ChildType>
void add(
        LogEntryPropertyWithChildren& _log_entry_property_with_children,
        const ChildType& _value)
{
    const auto log_entry_property = to_log_entry_property(_value);
    _log_entry_property_with_children.children[log_entry_property.type] = log_entry_property.value;
}

template <typename Type, typename = std::enable_if_t<not std::is_same<Type, LogEntryProperty>{}>>
LogEntryPropertyWithChildren& add(
        Properties& _properties,
        const Type& _value)
{
    return add(_properties, to_log_entry_property(_value));
}

void merge(Properties&, Properties&&);

} // namespace LibGrill::Logger
} // namespace LibGrill

#endif
